/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl>
 *
 */


#ifndef SSL_COMMON_H
 #define SSL_COMMON_H

#include <openssl/x509.h>
#include <openssl/ssl.h>

#define CIPHER_LIST "ALL:!ADH:!LOW:!EXP:@STRENGTH"
#define MAXCHAINDEPTH 99

extern SSL_CTX * xacml_io_ssl_ctx;
extern char * xacml_io_certCAPath;
extern char * xacml_io_certFilePub;
extern char * xacml_io_certFilePriv;
extern char * xacml_io_certOwner;


int xacml_io_init_OpenSSL(void);
void xacml_io_cleanup_OpenSSL(void);
void xacml_io_seed_prng(void);


char * xacml_io_get_hostname_to_match_client_cert(void);
int    xacml_io_set_hostname_to_match_client_cert(char * override_h_n);
void  xacml_io_free_hostname_to_match_client_cert(void);

int xacml_io_SSL_recv (SSL * ssl, void * data, int num);

SSL * xacml_io_SSL_client_connect (int opensocket, SSL_CTX * ctx);
SSL * xacml_io_SSL_server_accept (int opensocket, SSL_CTX * ctx);
int xacml_io_SSL_connect_shutdown (SSL * ssl);

SSL_CTX *xacml_io_setup_client_ctx(char * cafile, 
				   char * cadir, 
				   char * clientcert, 
				   char * clientkey);
SSL_CTX *xacml_io_setup_server_ctx(char * cafile,
				   char * cadir,
				   char * servercert,
				   char * serverkey);

int xacml_io_x509IsCA (X509 *cert);
char * xacml_io_x509_chain_to_dn(STACK_OF(X509) *chain);
int xacml_io_print_ssl_error_msg (SSL * ssl, int ret, int printmsg, int * error_code);

long xacml_io_post_connection_check(SSL *ssl, const char *host);

/************************/

#if 0
int
scas_gsi_callback_X509_verify_cert(
    X509_STORE_CTX *                    context,
    void *                              arg);
#endif

#ifndef DISABLE_PROXY_SUPPORT
/** Returns whether we want to run-time disable proxy support */
int xacml_io_get_disable_server_proxy_support(void);

/** Sets whether we want to run-time disable proxy support */
void xacml_io_set_disable_server_proxy_support(int value);
#endif /* DISABLE_PROXY_SUPPORT */

#endif /* COMMON_H */


